@@ -2,7 +2,7 @@
 C
 C   wght(1) is voxel extension x / voxel extension y,  i.e. zero in univariate situations
 C   wght(2) is voxel extension x / voxel extension z,  i.e. zero in univariate and bivariate situations
-      implicit logical(a-z)
+      implicit none
       integer kern
       double precision bw,wght(2),vol,sofw
       external sofw
@@ -10,7 +10,7 @@ C   wght(2) is voxel extension x / voxel extension z,  i.e. zero in univariate a
       RETURN
       END
       double precision function sofw(bw,kern,wght)
-      implicit logical(a-z)
+      implicit none
       integer kern
       double precision bw,wght(2)
       integer j1,j2,j3,dlw1,dlw2,dlw3,clw1,clw2,clw3,ih1,ih2,ih3
@@ -64,13 +64,13 @@ C
       END
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 C
-C   determine sum of location weights for a given geometry a(3) and given 
+C   determine sum of location weights for a given geometry a(3) and given
 C   bandwidth
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 C  Algorithmus zur Nullstellenbestimmung einer monotonen Funktion auf(0,\infty)
       subroutine gethani(x,y,kern,value,wght,eps,bw)
-      implicit logical(a-z)
+      implicit none
       integer kern
       double precision x,y,value,wght(2),eps,bw
       double precision fw1,fw2,fw3,z
@@ -87,7 +87,8 @@ C  Algorithmus zur Nullstellenbestimmung einer monotonen Funktion auf(0,\infty)
          y=y*y/x
          fw2=sofw(y,kern,wght)
       END DO
-      DO WHILE(min(fw2/value,value/fw1).gt.1.d0+eps)
+      DO WHILE(min(fw2/value,value/fw1).gt.1.d0+eps.and.
+     1             abs(y-x).gt.1d-6)
          z=x+(value-fw1)/(fw2-fw1)*(y-x)
          fw3=sofw(z,kern,wght)
          if(fw3.le.value) THEN
@@ -106,14 +107,14 @@ C  Algorithmus zur Nullstellenbestimmung einer monotonen Funktion auf(0,\infty)
           bw=y-(fw2-value)/(fw2-fw1)*(y-x)
       ENDIF
       RETURN
-      END  
+      END
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 C
 C   Calculate exceedence probabilities in awstestprop
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine exceed(x,n,z,nz,exprob)
-      implicit logical (a-z)
+      implicit none
       integer n,nz
       double precision x(n),z(nz),exprob(nz)
       integer i,j,k
@@ -136,4 +137,3 @@ C$OMP END PARALLEL
 C$OMP FLUSH(exprob)
       Return
       End
-      
